@@ -1,11 +1,16 @@
 #include <linux/export.h>
 #include <linux/types.h>
 #include <linux/io.h>
-#if defined(CONFIG_SYNO_LSP_ARMADA)
 #include <linux/spinlock.h>
 
 static DEFINE_RAW_SPINLOCK(__io_lock);
 
+/*
+ * Generic atomic MMIO modify.
+ *
+ * Allows thread-safe access to registers shared by unrelated subsystems.
+ * The access is protected by a single MMIO-wide lock.
+ */
 void atomic_io_modify_relaxed(void __iomem *reg, u32 mask, u32 set)
 {
 	unsigned long flags;
@@ -31,8 +36,11 @@ void atomic_io_modify(void __iomem *reg, u32 mask, u32 set)
 	raw_spin_unlock_irqrestore(&__io_lock, flags);
 }
 EXPORT_SYMBOL(atomic_io_modify);
-#endif  
 
+/*
+ * Copy data from IO memory space to "real" memory space.
+ * This needs to be optimized.
+ */
 void _memcpy_fromio(void *to, const volatile void __iomem *from, size_t count)
 {
 	unsigned char *t = to;
@@ -44,6 +52,10 @@ void _memcpy_fromio(void *to, const volatile void __iomem *from, size_t count)
 	}
 }
 
+/*
+ * Copy data from "real" memory space to IO memory space.
+ * This needs to be optimized.
+ */
 void _memcpy_toio(volatile void __iomem *to, const void *from, size_t count)
 {
 	const unsigned char *f = from;
@@ -55,6 +67,10 @@ void _memcpy_toio(volatile void __iomem *to, const void *from, size_t count)
 	}
 }
 
+/*
+ * "memset" on IO memory space.
+ * This needs to be optimized.
+ */
 void _memset_io(volatile void __iomem *dst, int c, size_t count)
 {
 	while (count) {
