@@ -350,6 +350,7 @@ struct mn10300_serial_port mn10300_serial_port_sif2 = {
 };
 #endif /* CONFIG_MN10300_TTYSM2 */
 
+
 /*
  * list of available serial ports
  */
@@ -366,6 +367,7 @@ struct mn10300_serial_port *mn10300_serial_ports[NR_UARTS + 1] = {
 	[NR_UARTS] = NULL,
 };
 
+
 /*
  * we abuse the serial ports' baud timers' interrupt lines to get the ability
  * to deliver interrupts to userspace as we use the ports' interrupt lines to
@@ -983,17 +985,17 @@ static int mn10300_serial_startup(struct uart_port *_port)
 	irq_set_chip(port->tm_irq, &mn10300_serial_pic);
 
 	if (request_irq(port->rx_irq, mn10300_serial_interrupt,
-			IRQF_DISABLED | IRQF_NOBALANCING,
+			IRQF_NOBALANCING,
 			port->rx_name, port) < 0)
 		goto error;
 
 	if (request_irq(port->tx_irq, mn10300_serial_interrupt,
-			IRQF_DISABLED | IRQF_NOBALANCING,
+			IRQF_NOBALANCING,
 			port->tx_name, port) < 0)
 		goto error2;
 
 	if (request_irq(port->tm_irq, mn10300_serial_interrupt,
-			IRQF_DISABLED | IRQF_NOBALANCING,
+			IRQF_NOBALANCING,
 			port->tm_name, port) < 0)
 		goto error3;
 	mn10300_serial_mask_ack(port->tm_irq);
@@ -1562,6 +1564,7 @@ static int __init mn10300_serial_init(void)
 
 __initcall(mn10300_serial_init);
 
+
 #ifdef CONFIG_MN10300_TTYSM_CONSOLE
 
 /*
@@ -1745,6 +1748,7 @@ static int mn10300_serial_poll_get_char(struct uart_port *_port)
 	return ch;
 }
 
+
 /*
  * Polled character transmission for the kernel debugger
  */
