@@ -30,15 +30,8 @@
 #define __DPC_H__
 
 #include "device.h"
-#include "wcmd.h"
 
-void RXvWorkItem(struct work_struct *work);
-
-void RXvMngWorkItem(struct work_struct *work);
-
-void RXvFreeRCB(struct vnt_rcb *pRCB, int bReAllocSkb);
-
-int RXbBulkInProcessData(struct vnt_private *, struct vnt_rcb *pRCB,
-	unsigned long BytesToIndicate);
+int vnt_rx_data(struct vnt_private *, struct vnt_rcb *,
+	unsigned long bytes_recieved);
 
 #endif /* __RXTX_H__ */
